﻿using System;
using System.IO;
using System.Text;
using Yahoo.Yui.Compressor;

namespace CompressCSS
{
	internal class Program
	{
		private static void Main(string[] args)
		{			
			var baseFolder = AppDomain.CurrentDomain.BaseDirectory;
			var themesFolder = Path.Combine(baseFolder, "themes");

			if (!Directory.Exists(themesFolder))
			{
				Console.WriteLine("Theme folder '{0}' does not exist.");
				return;
			}

			foreach (var combineFile in Directory.GetFiles(baseFolder, "*.combine"))
			{
				foreach (var themeFolder in Directory.GetDirectories(themesFolder))
				{
					var di = new DirectoryInfo(themeFolder);

					if ((di.Attributes & FileAttributes.Hidden) == FileAttributes.Hidden)
					{
						continue;
					}

					var cssFileName = string.Format("{0}.css", Path.GetFileNameWithoutExtension(combineFile));
					var combined = new StringBuilder();

					using (var sr = new StreamReader(combineFile))
					{
						var line = sr.ReadLine();

						while (!string.IsNullOrEmpty(line))
						{
							var file = line.StartsWith(@".\")
							           	? Path.Combine(baseFolder, Path.GetFileName(line))
							           	: (Path.IsPathRooted(line)
							           	   	? line
							           	   	: Path.Combine(themeFolder, line));

							if (File.Exists(file))
							{
								combined.Append(File.ReadAllText(file));
								Console.WriteLine("combined: '{0}'", file);
							}
							else
							{
								Console.WriteLine("not found (ignored): '{0}'", file);
							}

							line = sr.ReadLine();
						}
					}

					var compressCssPath = Path.Combine(themeFolder, cssFileName);
					var css = combined.ToString();

					if (!string.IsNullOrEmpty(css))
					{
						File.WriteAllText(compressCssPath, CssCompressor.Compress(css));
						Console.WriteLine("Compressed combined css to '{0}'.", compressCssPath);
					}
					else
					{
						Console.WriteLine("Css is empty.  Did not write '{0}'.", compressCssPath);
					}
				}
			}
		}
	}
}