﻿using System;
using System.IO;
using System.Text;
using Yahoo.Yui.Compressor;

namespace CompressJavascript
{
	internal class Program
	{
		private static Logger logger;

		private static void Main(string[] args)
		{
			var baseFolder = AppDomain.CurrentDomain.BaseDirectory;

			using (logger = new Logger())
			{
				CompressFolderJavascript(baseFolder);
				CombineFolderJavascript(baseFolder);
			}
		}

		private static void CompressFolderJavascript(string folder)
		{
			foreach (var file in Directory.GetFiles(folder, "*.js"))
			{
				if (file.EndsWith(".min.js", StringComparison.InvariantCultureIgnoreCase))
				{
					continue;
				}

				var minFile = Path.Combine(folder, string.Concat(Path.GetFileNameWithoutExtension(file), ".min.js"));

				if (File.Exists(minFile))
				{
					File.Delete(minFile);
				}

				logger.Log("minifiying: '{0}'", file);

				try
				{
					File.WriteAllText(minFile, new JavaScriptCompressor(File.ReadAllText(file)).Compress());
				}
				catch (Exception exception)
				{
					logger.Log("EXCEPTION: {0}", exception.Message);

					Environment.Exit(1);
				}
			}

			foreach (var directory in Directory.GetDirectories(folder))
			{
				CompressFolderJavascript(directory);
			}
		}

		private static void CombineFolderJavascript(string folder)
		{
			foreach (var combineFile in Directory.GetFiles(folder, "*.combine"))
			{
				var combined = new StringBuilder();

				using (var sr = new StreamReader(combineFile))
				{
					var line = sr.ReadLine();

					while (!string.IsNullOrEmpty(line))
					{
						var file = Path.IsPathRooted(line) ? line : Path.Combine(folder, line);

						if (File.Exists(file))
						{
							combined.Append(File.ReadAllText(file));
							logger.Log("combining: '{0}'", file);
						}
						else
						{
							logger.Log("not found (ignored): '{0}'", file);
						}

						line = sr.ReadLine();
					}
				}

				var combinedFile = Path.Combine(folder, string.Format("{0}.min.js", Path.GetFileNameWithoutExtension(combineFile)));

				if (File.Exists(combinedFile))
				{
					File.Delete(combinedFile);
				}

				File.WriteAllText(combinedFile, combined.ToString());

				logger.Log("combined: {0} -> {1}", combineFile, Path.GetFileName(combinedFile));
			}

			foreach (var directory in Directory.GetDirectories(folder))
			{
				CombineFolderJavascript(directory);
			}
		}
	}
}