﻿using System;
using System.Web.Mvc;

namespace Shuttle.Wiki.MVC
{
	public class SearchController : ShuttleController
	{
		public static string Name = "Search";
		private readonly IFindTask findTask;
		private readonly ISearchQuery searchQuery;

		public SearchController(IFindTask findTask, ISearchQuery searchQuery)
		{
			this.findTask = findTask;
			this.searchQuery = searchQuery;
		}

		[HttpGet]
		public ActionResult Page()
		{
			var page = Convert.ToInt32(RouteData.Id());

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				return PartialView(new ResultPageModel(page, searchQuery.Page(page)));
			}
		}

		[HttpGet]
		public ActionResult Find()
		{
			var searchText = RouteData.Id();

			var specification = new FindSpecification
			                    	{
			                    		SessionId = UserSession.SessionId,
			                    		SearchText = searchText
			                    	};

			int matches;

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				matches = findTask.Execute(specification);
			}

			return View(Views.Results, new SearchResultModel
			                           	{
			                           		SearchText = searchText,
			                           		MatchCount = matches,
			                           		PageCount = (matches/30) + (matches%30 > 0
			                           		                            	? 1
			                           		                            	: 0)
			                           	});
		}

		public static class Actions
		{
			public static readonly string Find = "Find";
		}

		public static class Views
		{
			public static readonly string Results = "Results";
		}
	}
}