﻿using System.Web.Mvc;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki.MVC
{
    public class SettingController : ShuttleController
    {
        private readonly ISystemSettingRepository repository;
        public static string Name = "Setting";

        public SettingController(ISystemSettingRepository repository)
        {
            this.repository = repository;
        }

        [HttpGet]
        [MemberAccess(MemberRole.Administrator)]
        public ActionResult Manage()
        {
            SystemSettings settings;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                settings = repository.Get();

                Guard.AgainstMissing<SystemSettings>(settings, "all");
            }

            return View(new SettingModel(Actions.Save, Name)
                            {
                                WikiName = settings.WikiName,
                                NoReplyEMailAddress = settings.NoReplyEMailAddress,
                                MemberRegistrationMode = settings.MemberRegistrationMode,
                                EMailDomain = settings.EMailDomain
                            });
        }

        [HttpPost]
        [MemberAccess(MemberRole.Administrator)]
        public ActionResult Save()
        {
            var form = Request.Form;

            using (var connection = DatabaseConnectionFactory.Create(WikiData.Source).BeginTransaction())
            {
                var settings = repository.Get();

                Guard.AgainstMissing<SystemSettings>(settings, "all");

                settings.WikiName = SettingView.WikiName.MapFrom(form);
                settings.NoReplyEMailAddress = SettingView.NoReplyEMailAddress.MapFrom(form);
                settings.MemberRegistrationMode = SettingView.MemberRegistrationMode.MapFrom(form);
                settings.EMailDomain = SettingView.EMailDomain.MapFrom(form);

                repository.Save(settings);

                RegisterSuccessMessage("The system settings have been succesfully saved.");

                connection.CommitTransaction();
            }

            return RedirectToAction(Actions.Manage);
        }

        public static class Actions
        {
            public static readonly string Manage = "Manage";
            public static readonly string Save = "Save";
        }
    }
}
