var validationAttached = false;
var validationSettings = { rules: {}, messages: {} };
var defaultAjaxTimeout = 15000;
var recentPagesTimer = null;

$(document).ready(function () {
    $('a:not(.no-wait)').on('click', pageLoading);

    $('a.help').on('click', function () { showHelp($(this).attr('key'), $(this).attr('title')); });

    $('button').button();

    $(document).ajaxError(
    function (event, request, errorType, errorThrown) {
        ajaxError(request, errorType, errorThrown);
    });

    $.ajaxSetup({ timeout: defaultAjaxTimeout });

    setMessageDialog();
    setConfirmationDialog();

    setEnterKeyCallbackOn('#search-text', function () { navigate('find', 'search', $('#search-text').val()); });

    renderImages();

    styleTables();
    styleFileInputs();
    wireRecentPages();
});

function renderImages() {
    $('.wiki-image-render').each(function (index, element) {
        $.ajax(
        {
            url: $(this).attr('url'),
            success: function (data) {
                $(element).replaceWith(data);
            },
            timeout: 3000,
            error: function (request, error) {
                $(element).html(error);
            }
        });
    });
}

function styleTables() {
    addToggleClass('tr', 'highlight');
    alternateRows('tr');

    $('tr th:nth-child(1),tr td:nth-child(1)').addClass('first-column')
}

function addToggleClass(selector, cssClass) {
    $(selector)
        .mouseover(function () { $(this).addClass(cssClass); })
        .mouseout(function () { $(this).removeClass(cssClass); });
}

function alternateRows(selector) {
    $(selector + ':even').addClass('even');
    $(selector + ':odd').addClass('odd');
}

function pageLoading() {
    $('#page-blocker').removeClass('ui-helper-hidden');
    $('#page-loading').removeClass('ui-helper-hidden');
}

function pageReady() {
    $('#page-blocker').addClass('ui-helper-hidden');
    $('#page-loading').addClass('ui-helper-hidden');
    $('button').removeAttr('disabled');
}

function navigate(action, controller, id) {
    pageLoading();

    window.location.href = url(action, controller, id);
}

function url(action, controller, id) {
    return domainUrl + controller + '/' + action + '/' + id;
}

function registerValidation(rules, messages) {
    $.extend(validationSettings.rules, rules);
    $.extend(validationSettings.messages, messages);
}

function attachValidation(form) {
    if (form == undefined) {
        form = $('#PropertiesForm');
    }

    if (!form.length) {
        throw 'Invalid or non-existent form used in attachValidation.';
    }

    var settings = form.validate({
        rules: {},
        messages: {},
        errorClass: 'form-element-error',
        errorElement: 'div',
        errorPlacement: function (error, element) {
            element.after(error);

            error.addClass('ui-helper-clearfix');
        }
    }).settings;

    $.extend(settings.rules, validationSettings.rules);
    $.extend(settings.messages, validationSettings.messages);

    settings = validationSettings;

    validationAttached = true;
}

function setEnterKeyCallbackOn(selector, callback) {
    $(selector).on('keyup', function (event) {
        if (event.keyCode == 13) {
            callback();
        }
    });
}

function setEnterKeyCallback(formId, callback) {
    if (typeof callback == "undefined") {
        return;
    }

    $('form').unbind('submit', pageLoading);

    $('#' + formId + ' fieldset > *').not('textarea').each(
         function () {
             $(this).on('keyup', function (event) {
                 if (event.keyCode == 13) {
                     callback();
                 }
             });
         });

    $('form:not(.no-wait)').on('submit', pageLoading);
}

function submitValidForm(formId) {
    var id = formId == undefined ? '#PropertiesForm' : '#' + formId;
    var form = $(id);

    if (form == undefined) {
        alert('Cannot find a form with id \'' + id + '\' to submit');
        return;
    }

    if (form.valid()) {
        form.submit();

        if (!form.hasClass('no-wait')) {
            pageLoading();
        }
    }
}

function isEmptyGuid(value) {
    var emptyGuid = /^(\{|\()?0{8}-(0{4}-){3}0{12}(\}|\))?$/;

    return emptyGuid.test(value);
}

function isValidGuid(value) {
    var validGuid = /^(\{|\()?[0-9a-fA-F]{8}-([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}(\}|\))?$/;

    return validGuid.test(value) && !isEmptyGuid(value);
}

function setConfirmationDialog() {
    $('#confirmation-dialog').dialog({
        modal: true,
        overlay: { opacity: 0.5, background: "#333" },
        autoOpen: false,
        title: 'Confirmation',
        width: '50%'
    });

    $('#confirmation-button-cancel').on('click', function (e) {
        $('#confirmation-dialog').dialog('close');
    });

    $('a.confirm-delete').on('click', function () {
        var href = $(this).attr("href");

        confirmation(
        {
            message: 'Are you sure that you want to delete the page?',
            callback: function () { pageLoading(); window.location = href; }
        });
        return false;
    });
}

function setMessageDialog() {
    $('#message-dialog').dialog({
        modal: true,
        overlay: { opacity: 0.5, background: "#333" },
        autoOpen: false,
        title: 'Message',
        width: '50%'
    });
    $('#message-button-close').on('click', function (e) {
        $('#message-dialog').dialog('close');
    });
}

function showMessage(message, title) {
    $('#message-dialog').dialog('option', 'title', title == undefined ? 'Message' : title);
    $('#message-container').html(message);
    $('#message-dialog').dialog('open');
}

function showHelp(key, title) {
    if (title != undefined && title != '') {
        $('#message-dialog').dialog('option', 'title', title);
    }
    else {
        $('#message-dialog').dialog('option', 'title', 'Help');
    }

    var help = $('#message-dialog').data(key);

    if (help == undefined || help.length == 0) {
        $('#message-container').html('');
        $('#message-container').addClass('server-call');
        $('#message-dialog').dialog('open');

        $.ajax(
        {
            url: url(key, 'Help'),
            success: function (data) {
                $('#message-container').html(data);
                $('#message-container').removeClass('server-call');
                $('#message-dialog').dialog('option', 'position', 'center');
                $('#message-dialog').data(key, data);
            },
            timeout: 3000,
            error: function (request, error) {
                if (error == 'timeout') {
                    $('#message-container').html('<p>The request has timed out.  Please try again.</p><p>If this problem persists please contact us.</p>');
                }
                else {
                    $('#message-container').html('<p>Error: ' + error + '.  Please try again.</p><p>If this problem persists please contact us.</p>');
                }
                $('#message-container').removeClass('server-call');
                $('#message-dialog').dialog('option', 'position', 'center');
            }
        });
    }
    else {
        $('#message-container').html(help);
        $('#message-dialog').dialog('option', 'position', 'center');
        $('#message-dialog').dialog('open');
    }
}

ajaxError = function (request, errorType, errorThrown) {
    pageReady();

    if (errorThrown != undefined && errorThrown != null && errorThrown.toString().indexOf('NS_ERROR_NOT_AVAILABLE') > 0) {
        return;
    }

    var message = errorThrown;

    if (request.status == 0) {
        message = '<p>There appears to be a network/connection problem.</p><p>If this behaviour persists please contact support.</p>';
    } else if (request.readyState == 4 && request.status == 404) {
        message = '<p>Requested url not found.</p>';
    } else if (request.readyState == 4 && request.status == 500) {
        message = '<p>Internal Server Error.</p><p>If this problem persists please contact support.</p><p>' + request.responseText + '</p>';
    } else if (errorType == 'parsererror') {
        message = '<p>Parse Error.</p><p>Parsing JSON Request failed.</p>';
    } else if (errorType == 'timeout') {
        message = '<p>The request has timed out.  Please try again.</p><p>If this problem persists please contact support.</p>';
    } else {
        message = '<p>Error: ' + errorThrown + '.  Please try again.</p><p>If this problem persists please contact support.</p>';
    }

    showMessage(message);
}

styleFileInputs = function () {
    $('.file-input').each(function () {
        var id = $(this).attr('id');
        var displayid = id + '-display';

        $(this).after('<input id=\'' + displayid + '\' readonly=readonly>');
        $('#' + displayid).on('click', function () { $('#' + id).click(); });
        $('#' + displayid).on('keydown', function () { $('#' + id).click(); });
        $('#' + id).on('change', function () {
            $('#' + displayid).val($('#' + id).val());
            $('#' + displayid).blur();
        });
    });
}

confirmation = function (options) {
    if (options == undefined || options.message == undefined || options.callback == undefined) {
        return;
    }

    var dialog = $('#confirmation-dialog');

    dialog.dialog('option', 'title', (options.title == undefined || options.title.length == 0) ? 'Confirmation' : options.title);

    $('#confirmation-message').html(options.message);

    $('#confirmation-button-ok').off('click');
    $('#confirmation-button-ok').on('click', function (e) {
        dialog.dialog('close');
        options.callback();
    });

    dialog.dialog('open');
}

function wireRecentPages() {
    addHoverClass('#recent-pages li.recent-page', 'ui-state-hover');

    $('#button-recent-pages')
        .button({ icons: { secondary: 'ui-icon-triangle-1-s'} })
        .bind('mouseleave', closeRecentPagesIntent)
        .bind('mouseenter', showRecentPages)
        .disableSelection()
        .show();

    $('#recent-pages')
        .bind('mouseleave', closeRecentPagesIntent)
        .bind('mouseenter', showRecentPages)
        .disableSelection();

    $('.recent-page').bind('click', function () {
        closeRecentPages();
        navigate($(this).text(), 'WikiPage');
    });
}

function closeRecentPagesIntent() {
    recentPagesTimer = setTimeout(function () {
        closeRecentPages();
    }, 1000);
}

closeRecentPages = function () {
    if (recentPagesTimer != null) {
        clearTimeout(recentPagesTimer);
        recentPagesTimer = null;
    }

    $('#recent-pages').hide();
}

showRecentPages = function () {
    if (recentPagesTimer != null) {
        clearTimeout(recentPagesTimer);
        recentPagesTimer = null;
    }

    $('#recent-pages').show();
    $('#recent-pages').position(
        {
            my: 'left top',
            at: 'left bottom',
            of: '#button-recent-pages',
            collision: 'flip'
        });
}

addHoverClass = function (selector, cssClass) {
    addHoverClassObject($(selector), cssClass);
}

addHoverClassObject = function (obj, cssClass) {
    obj
        .mouseover(function () { $(this).addClass(cssClass); })
        .mouseleave(function () { $(this).removeClass(cssClass); });
}
