using System.Linq;
using NUnit.Framework;

namespace Shuttle.Wiki.Test
{
    public class WikiMarkupTest : Fixture
    {
        [Test]
        public void Should_be_able_to_get_multiple_sections_from_wiki_markup_with_no_first_heading()
        {
            const string TEXT_PART1 = "some markup\r\n";
            const string TEXT_PART2 = "! Heading 1\r\nfirst paragraph\r\n";
            const string TEXT_PART3 = "!! Heading 2\r\nsecond paragraph\r\n";

            var markup = new WikiMarkup(TEXT_PART1)
                .Append(TEXT_PART2)
                .Append(TEXT_PART3);

            var sections = markup.GetSections();

            Assert.AreEqual(3, sections.Count());
        }

        [Test]
        public void Should_be_able_to_get_multiple_sections_from_wiki_markup_with_first_heading()
        {
            const string TEXT_PART1 = "! Heading 1\r\nsome markup\r\n";
            const string TEXT_PART2 = "! Heading 2\r\nfirst paragraph\r\n";
            const string TEXT_PART3 = "!! Heading 3\r\nsecond paragraph\r\n";

            var markup = new WikiMarkup(TEXT_PART1)
                .Append(TEXT_PART2)
                .Append(TEXT_PART3);

            var sections = markup.GetSections();

            Assert.AreEqual(3, sections.Count());
        }

        [Test]
        public void Should_be_able_to_get_single_section_from_wiki_markup()
        {
            const string TEXT = "some markup";

            var markup = new WikiMarkup(TEXT);

            var sections = markup.GetSections();

            Assert.AreEqual(1, sections.Count());
            Assert.IsTrue(sections.ElementAt(0).BodyMarkup.Contains(TEXT));
        }
    }
}