using System.Data;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class MemberMapper : IDataRowMapper<Member>
    {
        private readonly ICryptographyService cryptographyService;
    	private readonly IApplicationConfiguration applicationConfiguration;

    	public MemberMapper(ICryptographyService cryptographyService, IApplicationConfiguration applicationConfiguration)
        {
        	this.cryptographyService = cryptographyService;
        	this.applicationConfiguration = applicationConfiguration;
        }

    	public MappedRow<Member> Map(DataRow row)
        {
    	    var newEMailAddress = MemberColumns.NewEMailAddress.MapFrom(row);

    	    var member = new Member(MemberColumns.Id.MapFrom(row))
            {
                Name = MemberColumns.Name.MapFrom(row),
                EMailAddress = new EMailAddress(MemberColumns.EMailAddress.MapFrom(row)),
                NewEMailAddress = new EMailAddress(newEMailAddress),
                RegistrationDate = MemberColumns.RegistrationDate.MapFrom(row),
                ActivationDate = MemberColumns.ActivationDate.MapFrom(row),
                Role = MemberColumns.Role.MapFrom(row)
            };

    		var encrypted = MemberColumns.Password.MapFrom(row);

			if (!string.IsNullOrEmpty(encrypted))
			{
				member.SetPasswordEncrypted(encrypted, cryptographyService, applicationConfiguration);
			}

            return new MappedRow<Member>(row, member);
        }
   }
}