﻿using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
    public class MemberRepository : Repository, IMemberRepository
    {
        private readonly IDataRepository<Member> repository;

        public MemberRepository(IDataRepository<Member> repository)
        {
            this.repository = repository;
        }

        public Member Get(Guid id)
        {
            return repository.FetchItemUsing(WikiData.Source, MemberTableAccess.Get(id));
        }

        public void Add(Member entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.Add(entity));
        }

        public void Remove(Member entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.Remove(entity));
        }

        public Member GetByName(string name)
        {
            return repository.FetchItemUsing(WikiData.Source, MemberTableAccess.GetByName(name));
        }

        public bool ContainsName(string name)
        {
            return DatabaseGateway.GetScalarUsing<int>(WikiData.Source, MemberTableAccess.ContainsName(name)) == 1;
        }

        public void SaveActivation(Member member)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.Activated(member));
        }

        public void SavePassword(Member member)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.SavePassword(member));
        }

        public int Count
        {
            get { return DatabaseGateway.GetScalarUsing<int>(WikiData.Source, MemberTableAccess.Count()); }
        }

        public void SaveNewEMailAddress(Member member)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.SaveNewEMailAddress(member));
        }

        public Member GetByEMailAddress(string address)
        {
            return repository.FetchItemUsing(WikiData.Source, MemberTableAccess.GetByEMailAddress(address));
        }

        public void SaveEMailActivation(Member member)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, MemberTableAccess.SaveEMailActivation(member));
        }
    }
}