using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
	public static class WikiImageQueries
	{
		private const string TableName = "WikiImage";

		public static IQuery GetName(Guid id)
		{
			return SelectBuilder
				.Select(WikiImageColumns.Name)
				.Where(WikiImageColumns.Id).EqualTo(id)
				.From(TableName);
		}

		public static IQuery HasName(string name)
		{
			return ContainsBuilder
				.Where(WikiImageColumns.Name).EqualTo(name)
				.In(TableName);
		}

		public static ISelectBuilderSelect SelectClause()
		{
			return SelectBuilder
				.Select(WikiImageColumns.Id)
				.With(WikiImageColumns.Name)
				.With(WikiImageColumns.FileExtension);
		}

		public static IQuery Search(string name)
		{
			return SelectClause()
				.Where(WikiImageColumns.Name).Like(string.Format("%{0}%", name))
				.From(TableName);
		}

		public static IQuery GetForRendering(string name)
		{
			return SelectClause()
				.Where(WikiImageColumns.Name).EqualTo(name)
				.From(TableName);
		}
	}
}