using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
    public static class MemberTableAccess
    {
        private const string TableName = "Member";

		public static IQuery Get(Guid id)
        {
            return SelectClause()
                .Where(MemberColumns.Id).EqualTo(id)
                .From(TableName);
        }

        private static ISelectBuilderSelect SelectClause()
        {
            return SelectBuilder
                .Select(MemberColumns.Id)
                .With(MemberColumns.Password)
                .With(MemberColumns.Name)
                .With(MemberColumns.EMailAddress)
                .With(MemberColumns.NewEMailAddress)
                .With(MemberColumns.ActivationDate)
                .With(MemberColumns.RegistrationDate)
                .With(MemberColumns.Role);
        }

		public static IQuery Add(Member member)
        {
            return InsertBuilder
                    .Insert()
                    .Add(MemberColumns.Id).WithValue(member.Id)
                    .Add(MemberColumns.Password).WithValue(member.PasswordEncrypted)
                    .Add(MemberColumns.Name).WithValue(member.Name)
                    .Add(MemberColumns.EMailAddress).WithValue(member.EMailAddress.ToString())
                    .Add(MemberColumns.NewEMailAddress).WithValue(member.NewEMailAddress.ToString())
                    .Add(MemberColumns.RegistrationDate).WithValue(member.RegistrationDate)
                    .Add(MemberColumns.ActivationDate).WithValue(member.ActivationDate)
                    .Add(MemberColumns.Role).WithValue(member.Role)
                    .Into(TableName);
        }

		public static IQuery Remove(Member member)
        {
            return DeleteBuilder
                .Where(MemberColumns.Id).EqualTo(member.Id)
                .From(TableName);
        }

		public static IQuery GetByName(string name)
        {
            return SelectClause()
                    .Where(MemberColumns.Name).EqualTo(name)
                    .From(TableName);
        }

		public static IQuery ContainsName(string name)
        {
            return ContainsBuilder
                .Where(MemberColumns.Name).EqualTo(name)
                .In(TableName);
        }

		public static IQuery Activated(Member member)
        {
            return UpdateBuilder
                .Update()
                .Set(MemberColumns.ActivationDate).ToValue(member.ActivationDate)
                .Where(MemberColumns.Id).EqualTo(member.Id)
				.In(TableName);
        }

		public static IQuery SavePassword(Member member)
        {
            return UpdateBuilder
                .Update()
                .Set(MemberColumns.Password).ToValue(member.PasswordEncrypted)
                .Where(MemberColumns.Id).EqualTo(member.Id)
				.In(TableName);
        }

        public static IQuery Count()
        {
            return RawQuery.CreateFrom("select count(*) from Member");
        }

        public static IQuery SaveNewEMailAddress(Member member)
        {
            return UpdateBuilder
               .Update()
               .Set(MemberColumns.NewEMailAddress).ToValue(member.NewEMailAddress.ToString())
               .Where(MemberColumns.Id).EqualTo(member.Id)
               .In(TableName);
        }

        public static IQuery GetByEMailAddress(string address)
        {
            return SelectClause()
                   .Where(MemberColumns.EMailAddress).EqualTo(address)
                   .From(TableName);
        }

        public static IQuery SaveEMailActivation(Member member)
        {
            return UpdateBuilder
               .Update()
               .Set(MemberColumns.EMailAddress).ToValue(member.EMailAddress.ToString())
               .Where(MemberColumns.Id).EqualTo(member.Id)
               .In(TableName);
        }
    }
}