using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
    public static class WikiImageTableAccess
    {
        private const string TableName = "WikiImage";

		public static IQuery Get(Guid id)
        {
            return SelectClause()
                .Where(WikiImageColumns.Id).EqualTo(id)
                .From(TableName);
        }

		private static ISelectBuilderSelect SelectClause()
		{
			return SelectBuilder
				.Select(WikiImageColumns.Id)
				.With(WikiImageColumns.Name)
				.With(WikiImageColumns.FileExtension);
		}

		public static IQuery Add(WikiImage WikiImage)
        {
            return InsertBuilder
                    .Insert()
                    .Add(WikiImageColumns.Id).WithValue(WikiImage.Id)
                    .Add(WikiImageColumns.Name).WithValue(WikiImage.Name)
                    .Add(WikiImageColumns.FileExtension).WithValue(WikiImage.FileExtension)
                    .Into(TableName);
        }

		public static IQuery Remove(WikiImage WikiImage)
        {
            return DeleteBuilder
                .Where(WikiImageColumns.Id).EqualTo(WikiImage.Id)
                .From(TableName);
        }

		public static IQuery GetByName(string name)
        {
            return SelectClause()
                    .Where(WikiImageColumns.Name).EqualTo(name)
                    .From(TableName);
        }

		public static IQuery ContainsName(string name)
        {
            return ContainsBuilder
                .Where(WikiImageColumns.Name).EqualTo(name)
                .In(TableName);
        }

		public static IQuery Contains(WikiImage entity)
    	{
			return ContainsBuilder
				.Where(WikiImageColumns.Id).EqualTo(entity.Id)
				.In(TableName);
		}

		public static IQuery Save(WikiImage entity)
    	{
    		return UpdateBuilder
    			.Update()
    			.Set(WikiImageColumns.Name).ToValue(entity.Name)
    			.Set(WikiImageColumns.FileExtension).ToValue(entity.FileExtension)
    			.Where(WikiImageColumns.Id).EqualTo(entity.Id)
				.In(TableName);
    	}
    }
}