using System;
using System.Text;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
	public static class WikiPageTableAccess
	{
		private const string TableName = "WikiPage";

		public static IQuery Get(Guid id)
		{
			return SelectClause()
				.Where(WikiPageColumns.Id).EqualTo(id)
				.From(TableName);
		}

		private static ISelectBuilderSelect SelectClause()
		{
			return SelectBuilder
				.Select(WikiPageColumns.Id)
				.With(WikiPageColumns.PageName);
		}

		public static IQuery Add(WikiPage wikiPage)
		{
			return InsertBuilder
				.Insert()
				.Add(WikiPageColumns.Id).WithValue(wikiPage.Id)
				.Add(WikiPageColumns.PageName).WithValue(wikiPage.Name)
				.Add(WikiPageColumns.Html).WithValue(wikiPage.Render())
				.Into(TableName);
		}

		public static IQuery Remove(WikiPage wikiPage)
		{
			return DeleteBuilder
				.Where(WikiPageColumns.Id).EqualTo(wikiPage.Id)
				.From(TableName);
		}

		public static IQuery GetByName(string name)
		{
			return SelectClause()
				.Where(WikiPageColumns.PageName).EqualTo(name)
				.From(TableName);
		}

		public static IQuery ContainsName(string name)
		{
			return ContainsBuilder
				.Where(WikiPageColumns.PageName).EqualTo(name)
				.In(TableName);
		}

		public static IQuery SaveRendering(Guid pageId, string html)
		{
			return UpdateBuilder
				.Update()
				.Set(WikiPageColumns.Html).ToValue(html)
				.Where(WikiPageColumns.Id).EqualTo(pageId)
				.In(TableName);
		}

		public static IQuery Find(FindSpecification specification)
		{
			var sql = new StringBuilder();

			sql.Append("insert into SearchResult (SessionId, Id, ExpiresOn) ");

			sql.AppendLine(
				"select distinct top 300 @SessionId, PageId, @ExpiresOn from (select distinct top 300 PageId from WikiSection where HeadingMarkup like @SearchText or BodyMarkup like @SearchText union select distinct top 300 Id as PageId from WikiPage where PageName like @SearchText) ids");
			sql.AppendLine(
				"select min(Row) as SearchResultRowOffset, count(*) as ResultCount from SearchResult (nolock) where SessionId = @SessionId");

			var query = RawQuery.CreateFrom(sql.ToString());

			query.AddParameterValue(SearchResultColumns.SessionId, specification.SessionId);
			query.AddParameterValue(SearchResultColumns.ExpiresOn, DateTime.Now.AddHours(1));

			query.AddParameterValue(SearchResultColumns.SearchText, string.Format("%{0}%", specification.SearchText));

			return query;
		}

		public static IQuery Rename(string newPageName, string pageName)
		{
			return UpdateBuilder
				.Update()
				.Set(WikiPageColumns.PageName).ToValue(newPageName)
				.Where(WikiPageColumns.PageName).EqualTo(pageName)
				.In(TableName);
		}
	}
}