using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
    public static class WikiSectionTableAccess
    {
        private const string TableName = "WikiSection";

        public static IQuery Get(Guid id)
        {
            return SelectClause()
                .Where(WikiSectionColumns.Id).EqualTo(id)
                .From(TableName);
        }

        private static ISelectBuilderSelect SelectClause()
        {
            return SelectBuilder
                .Select(WikiSectionColumns.Id)
                .With(WikiSectionColumns.PageId)
                .With(WikiSectionColumns.SequenceNumber)
                .With(WikiSectionColumns.HeadingMarkup)
                .With(WikiSectionColumns.BodyMarkup);
        }

        public static IQuery Add(WikiPage wikiPage, WikiSection wikiSection, int sequenceNumber)
        {
            return InsertBuilder
                    .Insert()
                    .Add(WikiSectionColumns.Id).WithValue(wikiSection.Id)
                    .Add(WikiSectionColumns.PageId).WithValue(wikiPage.Id)
                    .Add(WikiSectionColumns.SequenceNumber).WithValue(sequenceNumber)
                    .Add(WikiSectionColumns.HeadingMarkup).WithValue(wikiSection.HeadingMarkup)
                    .Add(WikiSectionColumns.BodyMarkup).WithValue(wikiSection.BodyMarkup)
                    .Into(TableName);
        }

        public static IQuery Remove(WikiPage wikiPage)
        {
            return DeleteBuilder
                .Where(WikiSectionColumns.PageId).EqualTo(wikiPage.Id)
                .From(TableName);
        }

        public static IQuery GetSections(Guid wikiPageId)
        {
            return SelectClause()
                .Where(WikiSectionColumns.PageId).EqualTo(wikiPageId)
                .OrderBy(WikiSectionColumns.SequenceNumber).Ascending()
                .From(TableName);
        }
    }
}