﻿using System;
using Shuttle.Core.Data;

namespace Shuttle.Wiki
{
    public class WikiImageRepository : Repository, IWikiImageRepository
    {
        private readonly IDataRepository<WikiImage> repository;

        public WikiImageRepository(IDataRepository<WikiImage> repository)
        {
            this.repository = repository;
        }

        public WikiImage Get(Guid id)
        {
            return repository.FetchItemUsing(WikiData.Source, WikiImageTableAccess.Get(id));
        }

        public void Save(WikiImage entity)
        {
        	DatabaseGateway.ExecuteUsing(WikiData.Source, WikiImageTableAccess.Save(entity));
        }

    	public void Remove(WikiImage entity)
        {
            DatabaseGateway.ExecuteUsing(WikiData.Source, WikiImageTableAccess.Remove(entity));
        }

        public WikiImage GetByName(string name)
        {
            return repository.FetchItemUsing(WikiData.Source, WikiImageTableAccess.GetByName(name));
        }

        public bool ContainsName(string name)
        {
            return DatabaseGateway.GetScalarUsing<int>(WikiData.Source, WikiImageTableAccess.ContainsName(name)) == 1;
        }

    	public bool Contains(WikiImage entity)
    	{
			return DatabaseGateway.GetScalarUsing<int>(WikiData.Source, WikiImageTableAccess.Contains(entity)) == 1;
    	}

    	public void Add(WikiImage entity)
    	{
			DatabaseGateway.ExecuteUsing(WikiData.Source, WikiImageTableAccess.Add(entity));
		}
    }
}