using System;
using System.Text.RegularExpressions;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class EMailAddress
    {
        private static readonly Regex regex = new Regex(@"^(([^<>()[\]\\.,;:\s@\""]+"
                                                        + @"(\.[^<>()[\]\\.,;:\s@\""]+)*)|(\"".+\""))@"
                                                        + @"((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}"
                                                        + @"\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+"
                                                        + @"[a-zA-Z]{2,}))$");

        public EMailAddress(string address)
        {
            Guard.AgainstNullOrEmptyString(address, "address");

            if (!regex.IsMatch(address))
            {
                throw new ArgumentException(string.Format("'{0}' is not a valid e-mail address.", address));
            }

            Address = address;
        }

        public override string ToString()
        {
            return Address;
        }

        public string Address { get; private set; }
    }
}