using System;
using Shuttle.Core.Domain;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public enum MemberRole
    {
        Administrator = 0,
        Member = 1
    }

    public class Member : RowIdentity
    {
        public Member()
        {
        	Initialize();
        }

    	private void Initialize()
    	{
    		PasswordPlain = string.Empty;
    		PasswordEncrypted = string.Empty;
    	}

    	public Member(Guid id) : base(id)
        {
			Initialize();
		}

        public string Name { get; set; }
        public string Role { get; set; }
        public EMailAddress EMailAddress { get; set; }
        public EMailAddress NewEMailAddress { get; set; }
        public DateTime RegistrationDate { get; set; }
        public DateTime? ActivationDate { get; set; }

        public string PasswordPlain { get; private set; }
        public string PasswordEncrypted { get; private set; }

        public void SetPasswordPlain(string plain, ICryptographyService service, IApplicationConfiguration applicationConfiguration)
        {
            PasswordPlain = plain;
			PasswordEncrypted = service.TripleDESEncrypt(plain, applicationConfiguration.PasswordKey);
        }

        public void SetPasswordEncrypted(string encrypted, ICryptographyService service, IApplicationConfiguration applicationConfiguration)
        {
            PasswordEncrypted = encrypted;
			PasswordPlain = service.TripleDESDecrypt(encrypted, applicationConfiguration.PasswordKey);
        }

        public bool Activated
        {
            get { return ActivationDate.HasValue; }
        }

        public bool EMailAddressesAreTheSame
        {
            get { return EMailAddress.Address.Equals(NewEMailAddress.Address, StringComparison.InvariantCultureIgnoreCase); }
        }

        public void Activate()
        {
            ActivationDate = DateTime.Now;
        }

        public void MakeAdministrator()
        {
            Role = MemberRole.Administrator.ToString();
        }

        public bool Is(MemberRole role)
        {
            return (Role ?? string.Empty).Equals(role.ToString(), StringComparison.InvariantCultureIgnoreCase);
        }

        public void ActivateNewEMail()
        {
            EMailAddress = new EMailAddress(NewEMailAddress.Address);
        }
    }
}