using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace Shuttle.Wiki
{
    public class SystemSettings : ISystemSettings
    {
        private static readonly Type GuidType = typeof(Guid);

        private readonly Dictionary<string, string> settings = new Dictionary<string, string>();

        public void Add(string name, string value)
        {
            settings.Remove(name.ToLower());
            settings.Add(name.ToLower(), value);
        }

        public string NoReplyEMailAddress
        {
            get { return GetValue<string>("NoReplyEMailAddress"); }
            set { Add("NoReplyEMailAddress", value); }
        }

        public string WikiName
        {
            get { return GetValue<string>("WikiName"); }
            set { Add("WikiName", value); }
        }

        public string MemberRegistrationMode
        {
            get { return GetValue<string>("MemberRegistrationMode"); }
            set { Add("MemberRegistrationMode", value); }
        }

        public IEnumerable<KeyValuePair<string, string>> Settings
        {
            get { return new ReadOnlyCollection<KeyValuePair<string,string>>(settings.ToList()); }
        }

        public bool PrivateRegistration
        {
            get { return MemberRegistrationMode.Equals("private", StringComparison.InvariantCultureIgnoreCase); }
        }

        public string EMailDomain
        {
            get { return GetValue<string>("EMailDomain"); }
            set { Add("EMailDomain", value); }
        }

        public T GetValue<T>(string name, T @default = default(T))
        {
            if (!settings.ContainsKey(name.ToLower()))
            {
                return @default;
            }

            var value = settings[name.ToLower()];
            var underlyingSystemType = Nullable.GetUnderlyingType(typeof(T)) ?? typeof(T);

            return !typeof(Enum).IsAssignableFrom(underlyingSystemType)
                       ? typeof(T).IsAssignableFrom(GuidType)
                             ? (T)(object)new Guid(value)
                             : (T)Convert.ChangeType(value, underlyingSystemType)
                       : (T)Enum.Parse(underlyingSystemType, value);
        }
    }
}