using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class WikiMarkup
    {
        private readonly Regex headingExpression = new Regex(@"(?<!\w)!+\s.+\r*\n*",
                                                             RegexOptions.IgnoreCase | RegexOptions.Compiled);

        public WikiMarkup(string markup)
        {
            Guard.AgainstNullOrEmptyString(markup, "markup");

            Markup = string.Concat(markup.Trim(), Environment.NewLine);
        }

        public WikiMarkup()
        {
            Markup = string.Empty;
        }

        public string Markup { get; private set; }

        public IEnumerable<WikiSection> GetSections()
        {
            var result = new List<WikiSection>();

            var matches = headingExpression.Matches(Markup);

            if (matches.Count == 0)
            {
                result.Add(new WikiSection(string.Empty, Markup));
            }
            else
            {
                var pos = 0;

                var heading = string.Empty;

                foreach (Match match in matches)
                {
                    if (match.Index == 0)
                    {
                        heading = match.Value;
                        pos = match.Length;

                        continue;
                    }

                    result.Add(new WikiSection(heading, Markup.Substring(pos, match.Index - pos)));

                    heading = match.Value;
                    pos = match.Index + match.Length;
                }

                result.Add(pos == 0
                               ? new WikiSection(heading, Markup)
                               : new WikiSection(heading, Markup.Substring(pos, Markup.Length - pos)));
            }

            return result;
        }

        public WikiMarkup Append(string markup)
        {
            Guard.AgainstNullOrEmptyString(markup, "markup");

            Markup = markup.EndsWith(Environment.NewLine)
                         ? string.Concat(Markup, markup)
                         : string.Format("{0}{1}{2}", Markup, markup, Environment.NewLine);

            return this;
        }
    }
}