using Shuttle.Core.Infrastructure;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
	public class AddWikiImageHandler : MessageHandler, IMessageHandler<AddWikiImageCommand>
	{
		private readonly IWikiImageRepository repository;

		public AddWikiImageHandler(IWikiImageRepository repository)
		{
			Guard.AgainstNull(repository, "repository");

			this.repository = repository;
		}

		public void ProcessMessage(HandlerContext<AddWikiImageCommand> context)
		{
			var message = context.Message;

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				repository.Add(new WikiImage(message.Id)
				               	{
				               		Name = message.Name,
				               		FileExtension = message.FileExtension
				               	});
			}
		}

		public bool IsReusable
		{
			get { return true; }
		}
	}
}