using Shuttle.Core.Infrastructure;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
	public class ChangePasswordHandler : MessageHandler, IMessageHandler<ChangePasswordCommand>
	{
		private readonly IApplicationConfiguration applicationConfiguration;
		private readonly ICryptographyService cryptographyService;
		private readonly IMemberRepository repository;

		public ChangePasswordHandler(IMemberRepository repository, ICryptographyService cryptographyService, IApplicationConfiguration applicationConfiguration)
		{
			this.repository = repository;
			this.cryptographyService = cryptographyService;
			this.applicationConfiguration = applicationConfiguration;
		}

		public void ProcessMessage(HandlerContext<ChangePasswordCommand> context)
		{
			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				var member = repository.Get(context.Message.MemberId);

				if (member == null)
				{
					return;
				}

				member.SetPasswordPlain(context.Message.Password, cryptographyService, applicationConfiguration);

				repository.SavePassword(member);
			}
		}

		public bool IsReusable
		{
			get
			{
				return true;
			}
		}
	}
}