using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    public class CreateWikiPageHandler : MessageHandler, IMessageHandler<CreateWikiPageCommand>
    {
        private readonly IWikiRepository repository;

        public CreateWikiPageHandler(IWikiRepository repository)
        {
            this.repository = repository;
        }

        public void ProcessMessage(HandlerContext<CreateWikiPageCommand> context)
        {
            var markup = new WikiMarkup(context.Message.Markup);
            WikiPage page;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                page = repository.GetPageByName(context.Message.PageName);
            }

            var create = page == null;
            
            page = page ?? new WikiPage(context.Message.PageName);

            foreach (var section in markup.GetSections())
            {
                page.Add(section);
            }

            page.EnforceInvariants();

            using (var connection = DatabaseConnectionFactory.Create(WikiData.Source).BeginTransaction())
            {
                if (!create)
                {
                    repository.Save(page);
                }
                else
                {
                    repository.Add(page);
                }

                connection.CommitTransaction();
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }
    }
}