using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
	public class DeleteWikiPageHandler : MessageHandler, IMessageHandler<DeleteWikiPageCommand>
	{
		private readonly IWikiRepository repository;
		private readonly IUserSession userSession;

		public DeleteWikiPageHandler(IWikiRepository repository, IUserSession userSession)
		{
			this.repository = repository;
			this.userSession = userSession;
		}

		public void ProcessMessage(HandlerContext<DeleteWikiPageCommand> context)
		{
			using (var connection = DatabaseConnectionFactory.Create(WikiData.Source).BeginTransaction())
			{
				var page = repository.GetPageByName(context.Message.PageName);
				var historicalMarkup = page.GetMarkup();

				repository.Remove(page, historicalMarkup, userSession);

				connection.CommitTransaction();
			}
		}

		public bool IsReusable
		{
			get
			{
				return true;
			}
		}
	}
}