using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    public class DeleteWikiSectionHandler : MessageHandler, IMessageHandler<DeleteWikiSectionCommand>
    {
        private readonly IWikiRepository repository;
    	private readonly IUserSession userSession;

    	public DeleteWikiSectionHandler(IWikiRepository repository, IUserSession userSession)
        {
        	this.repository = repository;
        	this.userSession = userSession;
        }

    	public void ProcessMessage(HandlerContext<DeleteWikiSectionCommand> context)
        {
			WikiPage page;
			string historicalMarkup;

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				page = repository.GetPageByName(context.Message.PageName);
				historicalMarkup = page.GetMarkup();
			}

			page.Remove(context.Message.SectionId);

			page.NormalizeSections();
			page.EnforceInvariants();

			using (var connection = DatabaseConnectionFactory.Create(WikiData.Source).BeginTransaction())
			{
				repository.Save(page, historicalMarkup, userSession);

				connection.CommitTransaction();
			}
		}

        public bool IsReusable
        {
            get { return true; }
        }
    }
}