using Shuttle.Core.Infrastructure;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
	public class SaveWikiImageHandler : MessageHandler, IMessageHandler<SaveWikiImageCommand>
	{
		private readonly IWikiImageRepository repository;

		public SaveWikiImageHandler(IWikiImageRepository repository)
		{
			Guard.AgainstNull(repository, "repository");

			this.repository = repository;
		}

		public void ProcessMessage(HandlerContext<SaveWikiImageCommand> context)
		{
			var message = context.Message;

			using (DatabaseConnectionFactory.Create(WikiData.Source))
			{
				var image = repository.Get(message.Id);

				if (image == null)
				{
					return;
				}

				image.Name = message.Name;

				if (!string.IsNullOrEmpty(message.FileExtension))
				{
					image.FileExtension = message.FileExtension;
				}

				repository.Save(image);
			}
		}

		public bool IsReusable
		{
			get
			{
				return true;
			}
		}
	}
}