using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    public class SaveWikiSectionHandler : MessageHandler, IMessageHandler<SaveWikiSectionCommand>
    {
        private readonly IWikiRepository repository;
        private readonly IUserSession userSession;

        public SaveWikiSectionHandler(IWikiRepository repository, IUserSession userSession)
        {
            this.repository = repository;
            this.userSession = userSession;
        }

        public void ProcessMessage(HandlerContext<SaveWikiSectionCommand> context)
        {
            var markup = new WikiMarkup(context.Message.Markup);
            WikiPage page;
            string historicalMarkup;

            using (DatabaseConnectionFactory.Create(WikiData.Source))
            {
                page = repository.GetPageByName(context.Message.PageName);
                historicalMarkup = page.GetMarkup();
            }

            page.Replace(context.Message.SectionId, markup.GetSections());

            page.NormalizeSections();
            page.EnforceInvariants();

            using (var connection = DatabaseConnectionFactory.Create(WikiData.Source).BeginTransaction())
            {
                repository.Save(page, historicalMarkup, userSession);

                connection.CommitTransaction();
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }
    }
}