using System;
using System.Collections.Generic;
using System.Web;

namespace Shuttle.Wiki
{
    public class UserSession : IUserSession
    {
        private const string MemberNameKey = "MemberName";
        private const string MemberActivatedKey = "MemberActivated";
        private const string IsAdministratorKey = "IsAdministrator";
        private const string MemberIdKey = "MemberId";
        private const string SessionIdKey = "SessionId";
        private const string RecentPageNamesKey = "RecentPageNames";

        public bool IsLoggedOn
        {
            get { return Context.Session[MemberNameKey] != null; }
        }

        public Guid SessionId
        {
            get
            {
                var guid = Context.Session[SessionIdKey];

                if (guid == null)
                {
                    guid = Guid.NewGuid();

                    Context.Session.Add(SessionIdKey, guid);
                }

                return (Guid)guid;
            }
        }

        public int SearchResultRowOffset { get; set; }

        public bool IsAdministrator
        {
            get
            {
                var value = Context.Session[IsAdministratorKey];

                return value != null ? (bool)value : false;
            }
        }

        public Guid MemberId
        {
            get
            {
                return (Guid)Context.Session[MemberIdKey];
            }
        }

        public string MemberName
        {
            get
            {
                return (string)Context.Session[MemberNameKey];
            }
        }

        public void LogOn(Member member)
        {
            LogOff();

            Context.Session.Add(MemberNameKey, member.Name);
            Context.Session.Add(MemberActivatedKey, member.Activated);
            Context.Session.Add(IsAdministratorKey, member.Is(MemberRole.Administrator));
            Context.Session.Add(MemberIdKey, member.Id);
        }

        public void LogOff()
        {
            Context.Session.Clear();
        }

        public IEnumerable<string> RecentPageNames
        {
            get { return RecentPageNamesList(); }
        }

        public void AddRecentPageName(string name)
        {
            var list = RecentPageNamesList();

            if (list.Contains(name))
            {
                list.Remove(name);
            }

            if (list.Count == 10)
            {
                list.RemoveAt(9);
            }

            list.Insert(0, name);
        }

        private static List<string> RecentPageNamesList()
        {
            var list = Context.Session[RecentPageNamesKey];

            if (list == null)
            {
                list = new List<string>();

                Context.Session.Add(RecentPageNamesKey, list);
            }

            return (List<string>)list;
        }

        private static HttpContext Context
        {
            get
            {
                Guard();

                return HttpContext.Current;
            }
        }

        private static void Guard()
        {
            if (HttpContext.Current == null)
            {
                throw new NotSupportedException("HttpContext required.");
            }
        }
    }
}