using System.Collections.Generic;
using WikiPlex;
using WikiPlex.Compilation;
using WikiPlex.Compilation.Macros;

namespace Shuttle.Wiki
{
	public class WikiImageMacro : IMacro
	{
		public static string MacroId = "WikiImageMacro";

		public string Id
		{
			get { return MacroId; }
		}

		public IList<MacroRule> Rules
		{
			get
			{
				return new List<MacroRule>
				       {
                               new MacroRule(
                                   @"(?i)(<\[wiki-image:)(([^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkNoAltLeftAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(>\[wiki-image:)(([^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkNoAltRightAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(\[wiki-image:)(([^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkNoAlt},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(<\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkWithAltLeftAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(>\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkWithAltRightAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*\|https?://[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageWithLinkWithAlt},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(<\[wiki-image:)(([^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageLeftAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(>\[wiki-image:)(([^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageRightAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(\[wiki-image:)(([^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageNoAlign},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(<\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageLeftAlignWithAlt},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(>\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageRightAlignWithAlt},
                                           {3, ScopeName.Remove}
                                       }
                                   ),
                               new MacroRule(
                                   @"(?i)(\[wiki-image:)((?>[^\]\|]*\|[^\]\|]*))(\])",
                                   new Dictionary<int, string>
                                       {
                                           {1, ScopeName.Remove},
                                           {2, WikiScopeName.ImageNoAlignWithAlt},
                                           {3, ScopeName.Remove}
                                       }
                                   )
                           };
			}
		}
	}
}