﻿using System;
using System.Net.Mail;
using System.Text;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class MemberActivationTask : IMemberActivationTask
    {
        private readonly IMemberRepository repository;
        private readonly IEMailGateway emailGateway;
        private readonly IApplicationConfiguration applicationConfiguration;
        private readonly ISystemSettingRepository systemSettingRepository;

        public MemberActivationTask(IMemberRepository repository, IEMailGateway emailGateway, IApplicationConfiguration applicationConfiguration, ISystemSettingRepository systemSettingRepository)
        {
            this.repository = repository;
            this.emailGateway = emailGateway;
            this.applicationConfiguration = applicationConfiguration;
            this.systemSettingRepository = systemSettingRepository;
        }

        public void SendActivationEMail(Member member)
        {
            var settings = systemSettingRepository.Get();
            var body = new StringBuilder();

            var activationLink = string.Format("{0}member/activate/{1}", applicationConfiguration.SiteUrl, member.Id);

            body.AppendLine("<div id='e-mail-body' style='font-family: tahoma, verdana, helvetica, sans-serif; margin: 0; padding: 5px; height: 100%; color: #000000;'>");
            body.AppendFormat("<p>Hello {0},</p>", member.Name);
            body.AppendLine();
            body.AppendLine();
            body.AppendFormat("<p>Please activate your account by clicking <a href='{0}' style='color: #e27325; text-transform: lowercase;'>this activation link</a>.</p>", activationLink);
            body.AppendLine();
            body.AppendLine();
            body.AppendLine("<p>If the above link does not work copy the following into your browser's address bar and press enter:<br/>");
            body.AppendFormat("<span style='color: #e27325; text-transform: lowercase;'>{0}</span></p>", activationLink);
            body.AppendLine();
            body.AppendLine();
            body.AppendLine("<p>Regards,<br/>");
            body.AppendFormat("{0}</p>", settings.WikiName);
            body.AppendLine("</div>");

            emailGateway.SendMail(new MailMessage(settings.NoReplyEMailAddress,
                                                  member.NewEMailAddress.Address,
                                                  string.Format("{0} Member Activation : {1}", settings.WikiName, member.Name), body.ToString())
                                      {
                                          IsBodyHtml = true
                                      });
        }

        public void SendNewEMailAddressEMail(Member member)
        {
            var settings = systemSettingRepository.Get();
            var body = new StringBuilder();

            var activationLink = string.Format("{0}member/activateemail/{1}", applicationConfiguration.SiteUrl, member.Id);

            body.AppendLine("<div id='e-mail-body' style='font-family: tahoma, verdana, helvetica, sans-serif; margin: 0; padding: 5px; height: 100%; color: #000000;'>");
            body.AppendFormat("<p>Hello {0},</p>", member.Name);
            body.AppendLine();
            body.AppendLine();
            body.AppendFormat("<p>Please activate your new e-mail address by clicking <a href='{0}' style='color: #e27325; text-transform: lowercase;'>this activation link</a>.</p>", activationLink);
            body.AppendLine();
            body.AppendLine();
            body.AppendLine("<p>If the above link does not work copy the following into your browser's address bar and press enter:<br/>");
            body.AppendFormat("<span style='color: #e27325; text-transform: lowercase;'>{0}</span></p>", activationLink);
            body.AppendLine();
            body.AppendLine();
            body.AppendLine("<p>Regards,<br/>");
            body.AppendFormat("{0}</p>", settings.WikiName);
            body.AppendLine("</div>");

            emailGateway.SendMail(new MailMessage(settings.NoReplyEMailAddress,
                                                  member.NewEMailAddress.Address,
                                                  string.Format("{0} New E-Mail Address : {1}", settings.WikiName, member.Name), body.ToString())
                                      {
                                          IsBodyHtml = true
                                      });
        }

        public void SendCredentialsReminder(Member member)
        {
            var settings = systemSettingRepository.Get();
            var body = new StringBuilder();

            body.AppendLine("<div id='e-mail-body' style='font-family: tahoma, verdana, helvetica, sans-serif; margin: 0; padding: 5px; height: 100%; color: #000000;'>");
            body.AppendFormat("<p>Hello {0},</p>", member.Name);
            body.AppendLine();
            body.AppendLine();
            body.AppendFormat("<p>Your requested credentials for wiki '{0}' are as follows:</p>", settings.WikiName);
            body.AppendLine();
            body.AppendLine();
            body.AppendFormat("<p>Member name: {0}<br/>", member.Name);
            body.AppendFormat("Password: {0}</p>", member.PasswordPlain);
            body.AppendLine();
            body.AppendLine();
            body.AppendFormat("<p><a href='{0}' style='color: #e27325; text-transform: lowercase;'>log in to the wiki now</a></p>", string.Format("{0}member/credentials?name={1}", applicationConfiguration.SiteUrl, member.Name));
            body.AppendLine();
            body.AppendLine();
            body.AppendLine("<p>Regards,<br/>");
            body.AppendFormat("{0}</p>", settings.WikiName);
            body.AppendLine("</div>");

            emailGateway.SendMail(new MailMessage(settings.NoReplyEMailAddress,
                                                  member.NewEMailAddress.Address,
                                                  string.Format("{0} Credentials : {1}", settings.WikiName, member.Name), body.ToString())
            {
                IsBodyHtml = true
            });
        }

        public ActivationResult ActivateEMail(Guid memberId)
        {
            var member = repository.Get(memberId);

            if (member == null)
            {
                return ActivationResult.MemberNotFound;
            }

            if (member.EMailAddressesAreTheSame)
            {
                return ActivationResult.AlreadyActivated;
            }

            member.ActivateNewEMail();

            repository.SaveEMailActivation(member);

            return ActivationResult.Activated;
        }

        public ActivationResult Activate(Guid memberId)
        {
            var member = repository.Get(memberId);

            if (member == null)
            {
                return ActivationResult.MemberNotFound;
            }

            if (member.Activated)
            {
                return ActivationResult.AlreadyActivated;
            }

            member.Activate();

            repository.SaveActivation(member);

            return ActivationResult.Activated;
        }
    }
}