﻿using System;

namespace Shuttle.Wiki
{
    public enum MemberLoginResult
    {
        LoggedInSuccessfully = 0,
        MemberNotFound = 1,
        MemberNotYetActivated = 2,
        PasswordMismatch = 3
    }

	public class MemberLoginTask : IMemberLoginTask
	{
	    private readonly IMemberRepository repository;
		private readonly IUserSession userSession;

		public MemberLoginTask(IMemberRepository repository, IUserSession userSession)
		{
		    this.repository = repository;
			this.userSession = userSession;
		}

		public MemberLoginResult LogIn(string memberName, string password)
		{
			userSession.LogOff();

			var member = repository.GetByName(memberName);

			if (member == null)
			{
				return MemberLoginResult.MemberNotFound;
			}

            if (!member.Activated)
            {
                return MemberLoginResult.MemberNotYetActivated;
            }

		    if (!string.IsNullOrEmpty(member.PasswordPlain))
			{
				if (!member.PasswordPlain.Equals(password, StringComparison.InvariantCultureIgnoreCase))
				{
					return MemberLoginResult.PasswordMismatch;
				}
			}

			userSession.LogOn(member);

			return MemberLoginResult.LoggedInSuccessfully;
		}
	}
}