using System;
using System.Web.Mvc;

namespace Shuttle.Wiki
{
    public static class ViewDataDictionaryExtensions
    {
        private const string TitleKey = "Title";
        private const string HelpTopicKey = "HelpTopic";
        private const string HelpTitleKey = "HelpTitle";
        private const string UserSessionKey = "UserSession";

        public static string GetHelpTitle(this ViewDataDictionary data)
        {
            return HasHelpTitle(data) ? (string)data[HelpTitleKey] : "Help";
        }

        public static string GetHelpTopic(this ViewDataDictionary data)
        {
            return HasHelpTopic(data) ? (string)data[HelpTopicKey] : string.Empty;
        }

        public static bool HasTitle(this ViewDataDictionary data)
        {
            return data.ContainsKey(TitleKey);
        }

        public static bool HasHelpTitle(this ViewDataDictionary data)
        {
            return data.ContainsKey(HelpTitleKey);
        }

        public static bool HasHelpTopic(this ViewDataDictionary data)
        {
            return data.ContainsKey(HelpTopicKey);
        }

        public static void SetHelpTitle(this ViewDataDictionary data, string title)
        {
            if (data.ContainsKey(HelpTitleKey))
            {
                data.Remove(HelpTitleKey);
            }

            data.Add(HelpTitleKey, title);
        }

        public static void SetHelpTopic(this ViewDataDictionary data, string topic)
        {
            if (data.ContainsKey(HelpTopicKey))
            {
                data.Remove(HelpTopicKey);
            }

            data.Add(HelpTopicKey, topic);
        }

        public static void SetUserSession(this ViewDataDictionary data, IUserSession userSession)
        {
            if (!data.ContainsKey(UserSessionKey))
            {
                data.Add(UserSessionKey, userSession);
            }
        }

        public static IUserSession GetUserSession(this ViewDataDictionary data)
        {
            if (!data.ContainsKey(UserSessionKey))
            {
                throw new InvalidOperationException("No UserSession has been set on the ViewData.  Ensure that your controller inherits from ShuttleController.");
            }

            return (IUserSession)data[UserSessionKey];
        }

    }
}