using System;
using System.Web.Mvc;

namespace Shuttle.Wiki
{
    [AttributeUsage(AttributeTargets.Method | AttributeTargets.Class, AllowMultiple = false)]
    public class MemberAccessAttribute : ActionFilterAttribute
    {
        public MemberAccessAttribute() : this(MemberRole.Member)
        {
        }

        public MemberAccessAttribute(MemberRole role)
        {
            Role = role;
        }

        public MemberRole Role { get; private set;}

        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            var controller = filterContext.Controller as ShuttleController;

            if (controller != null && controller.UserSession.IsLoggedOn && Role == MemberRole.Administrator ? controller.UserSession.IsAdministrator : true)
            {
                return;
            }

            var ajaxContentType = AjaxContentType.None;

            if (filterContext.Controller.ViewData.ContainsKey(AjaxAttribute.ViewDataKey))
            {
                ajaxContentType = (AjaxContentType)filterContext.Controller.ViewData[AjaxAttribute.ViewDataKey];
            }

            if (filterContext.HttpContext.Request.IsAjaxRequest() || ajaxContentType != AjaxContentType.None)
            {
                var loginRequiredMessage = string.Format("Your session has expired.  Please <a href='{0}'>click here</a> to login again.", controller.ApplicationConfiguration.CredentialsUrl);

                switch (ajaxContentType)
                {
                    case AjaxContentType.Json:
                        {
                            filterContext.Result = new JsonResult
                            {
                                Data = new
                                {
                                    message = loginRequiredMessage,
                                    success = false
                                }
                            };

                            break;
                        }
                    default:
                        {
                            filterContext.Result = new ContentResult
                            {
                                ContentType = "text/plain",
                                Content = loginRequiredMessage
                            };

                            break;
                        }
                }

                if (ajaxContentType != AjaxContentType.Json)
                {
                    filterContext.HttpContext.Response.Status =
                        "500 " + loginRequiredMessage
                                    .Replace("\r", " ")
                                    .Replace("\n", " ");
                }
            }
            else
            {
                filterContext.Result = new RedirectResult(controller.ApplicationConfiguration.CredentialsUrl);
            }
        }
    }
}