using System.Web.Mvc;
using Shuttle.Core.Infrastructure;

namespace Shuttle.Wiki
{
    public class ShuttleHandleErrorAttribute : HandleErrorAttribute
    {
        public override void OnException(ExceptionContext filterContext)
        {
			Log.Error(filterContext.Exception.CompactMessages());

            var ajaxContentType = AjaxContentType.None;

            if (filterContext.Controller.ViewData.ContainsKey(AjaxAttribute.ViewDataKey))
            {
                ajaxContentType = (AjaxContentType)filterContext.Controller.ViewData[AjaxAttribute.ViewDataKey];
            }

            if (filterContext.HttpContext.Request.IsAjaxRequest() || ajaxContentType != AjaxContentType.None)
            {
                switch (ajaxContentType)
                {
                    case AjaxContentType.Json:
                        {
                            filterContext.Result = new JsonResult
                            {
                                Data = new
                                {
                                    message = filterContext.Exception.Message,
                                    success = false
                                }
                            };

                            break;
                        }
                    default:
                        {
                            filterContext.Result = new ContentResult
                            {
                                ContentType = "text/plain",
                                Content = filterContext.Exception.Message
                            };

                            break;
                        }
                }

                if (ajaxContentType != AjaxContentType.Json)
                {
                    filterContext.HttpContext.Response.Status =
                        "500 " + filterContext.Exception.Message
                                    .Replace("\r", " ")
                                    .Replace("\n", " ");
                }

                filterContext.HttpContext.Response.TrySkipIisCustomErrors = true;
                filterContext.ExceptionHandled = true;
            }
            else
            {
                base.OnException(filterContext);
            }
        }
    }
}