using System;
using System.Collections.Generic;
using System.Security.Principal;
using Shuttle.Core.Infrastructure;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    public class MemoryBus : IServiceBus, IActiveState
    {
        private static readonly Dictionary<string, object> handlers = new Dictionary<string, object>();

        public void Dispose()
        {
        }

    	public void MergeOutgoingHeaders()
    	{
    	}

    	public void ResetOutgoingHeaders()
        {
            throw new NotImplementedException();
        }

        public TransportMessage CreateTransportMessage(IMessage message)
        {
            var result = new TransportMessage();

            var identity = WindowsIdentity.GetCurrent();

            result.PrincipalIdentityName = identity != null
                                               ? identity.Name
                                               : WindowsIdentity.GetAnonymous().Name;

            if (result.SendDate == DateTime.MinValue)
            {
                result.SendDate = DateTime.Now;
            }

            result.Message = null;

            return result;
        }

        public TransportMessage Send(TransportMessage transportMessage, IMessage message1)
        {
            return null;
        }

        public TransportMessage Send(IMessage message)
        {
            Guard.AgainstNull(message, "message");

            var name = message.GetType().Name;

            if (!handlers.ContainsKey(name))
            {
                throw new ApplicationException(string.Format("Could not find a message handler for message type '{0}'",
                                                             message.GetType().FullName));
            }

            var handler = handlers[name];

            var contextType = typeof (HandlerContext<>).MakeGenericType(new[] {message.GetType()});
            var method = handler.GetType().GetMethod("ProcessMessage", new[] {contextType});

            if (method != null)
            {
                method.Invoke(handler, new[]
                                           {
                                               Activator.CreateInstance(contextType,
                                                                        new object[]
                                                                            {
                                                                                this,
                                                                                CreateTransportMessage(message),
                                                                                message,
                                                                                this
                                                                            })
                                           });
            }

            return null;
        }

        public TransportMessage Send(IMessage message, string uri)
        {
            throw new NotImplementedException();
        }

        public TransportMessage Send(IMessage message, IQueue queue)
        {
            throw new NotImplementedException();
        }

        public TransportMessage SendLocal(IMessage message)
        {
            throw new NotImplementedException();
        }

        public TransportMessage SendReply(IMessage message)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<string> Publish(IEventMessage message)
        {
            throw new NotImplementedException();
        }

        public IServiceBus Start()
        {
            return this;
        }

        public void Stop()
        {
        }

    	public string CorrelationId
    	{
    		get { return null; }
    		set { }
    	}

    	public string OutgoingCorrelationId
    	{
    		get { return null; }
    		set { }
    	}

    	public List<TransportHeader> OutgoingHeaders
        {
            get { throw new NotImplementedException(); }
        }

        public TransportMessage TransportMessageReceived
        {
            get { throw new NotImplementedException(); }
            set { throw new NotImplementedException(); }
        }

        public bool Started
        {
            get { return true; }
        }

        public IServiceBusConfiguration Configuration
        {
            get { throw new NotImplementedException(); }
        }

        public IServiceBusEvents Events
        {
            get { throw new NotImplementedException(); }
        }

        public static void InitializeWith(IResolver resolver)
        {
            var generic = typeof (IMessageHandler<>);

            foreach (var handler in Resolver.Instance.ResolveAssignable(generic))
            {
                foreach (var type in handler.GetType().GetInterfaces())
                {
                    if (!type.IsGenericType || type.GetGenericTypeDefinition() != generic)
                    {
                        continue;
                    }

                    handlers.Add(type.GetGenericArguments()[0].Name, handler);
                }
            }
        }

        public bool Active
        {
            get { return true; }
        }
    }
}