﻿using System;
using System.Web.Mvc;
using System.Web.Routing;
using Shuttle.Core.Data;
using Shuttle.Core.Infrastructure;
using Shuttle.ESB.Core;

namespace Shuttle.Wiki
{
    [ShuttleHandleError]
    public class ShuttleController : Controller
    {
        public IServiceBus Bus { get; set; }
        public IDatabaseConnectionFactory DatabaseConnectionFactory { get; set; }
        public IUserSession UserSession { get; set; }
        public IApplicationConfiguration ApplicationConfiguration { get; set; }
        public ISystemSettingRepository SystemSettingRepository { private get; set; }

        protected override void Initialize(RequestContext requestContext)
        {
            base.Initialize(requestContext);

            ViewData.SetUserSession(UserSession);
        }

        protected void RegisterFailureMessage(string message)
        {
            var messages = TempData.GetFailureMessages();

            messages.Add(message);

            TempData.SetFailureMessages(messages);
        }

        protected void RegisterSuccessMessage(string message)
        {
            var messages = TempData.GetSuccessMessages();

            messages.Add(message);

            TempData.SetSuccessMessages(messages);
        }

        protected override void HandleUnknownAction(string actionName)
        {
            try
            {
                View(actionName).ExecuteResult(ControllerContext);
            }
            catch (InvalidOperationException ex)
            {
                ViewData["actionName"] = actionName;
                ViewData["controllerName"] = RouteData.ControllerName();
                ViewData["message"] = ex.Message;
                ViewData.Model = new HandleErrorInfo(ex, RouteData.ControllerName(), actionName);

                View("Error").ExecuteResult(ControllerContext);
            }
        }

        protected ActionResult JsonError()
        {
            return JsonError(string.Empty);
        }

        protected ActionResult JsonError(string message)
        {
            return Json(
                new
                {
                    success = false,
                    message
                });
        }

        protected ActionResult JsonSuccess()
        {
            return Json(
                new
                {
                    success = true
                });
        }
        
        protected RedirectToRouteResult WikiIndex()
        {
            return RedirectToAction("Index", "WikiPage");
        }

        protected SystemSettings SystemSettings()
        {
            Guard.AgainstNullDependency(SystemSettingRepository, "SystemSettingRepository");

            return SystemSettingRepository.Get();
        }
    }
}